Class: Faulty::Storage::Interface

Inherits:
Object
  • Object
show all
Defined in:
lib/faulty/storage/interface.rb

Overview

The interface required for a storage backend implementation

This is for documentation only and is not loaded

Instance Method Summary collapse

Instance Method Details

#clearvoid

This method returns an undefined value.

Reset all circuits

Some implementions may clear circuits on a best-effort basis since all circuits may not be known.

Raises:

  • NotImplementedError If the storage backend does not support clearing.



182
183
184
# File 'lib/faulty/storage/interface.rb', line 182

def clear
  raise NotImplementedError
end

#close(circuit) ⇒ Boolean

Set the circuit state to closed

If multiple parallel processes close the circuit simultaneously, close may be called more than once. If so, this method should return true only once, when the circuit transitions from open to closed.

If the backend does not support locking or atomic operations, then it may always return true, but that could result in duplicate close notifications.

Raises:

  • (NotImplementedError)


98
99
100
# File 'lib/faulty/storage/interface.rb', line 98

def close(circuit)
  raise NotImplementedError
end

#entry(circuit, time, success, status) ⇒ Status?

Add a circuit run entry to storage

The backend may choose to store this in whatever manner it chooses as long as it can implement the other read methods.

Raises:

  • (NotImplementedError)


43
44
45
# File 'lib/faulty/storage/interface.rb', line 43

def entry(circuit, time, success, status)
  raise NotImplementedError
end

#fault_tolerant?Boolean

Can this storage backend raise an error?

If the storage backend returns false from this method, it will be wrapped in a FaultTolerantProxy, otherwise it will be used as-is.

Raises:

  • (NotImplementedError)


192
193
194
# File 'lib/faulty/storage/interface.rb', line 192

def fault_tolerant?
  raise NotImplementedError
end

#get_options(circuit) ⇒ Hash

Get the options stored for circuit

They should be returned exactly as given by #set_options

Raises:

  • (NotImplementedError)


15
16
17
# File 'lib/faulty/storage/interface.rb', line 15

def get_options(circuit)
  raise NotImplementedError
end

#history(circuit) ⇒ Array<Array>

Get the entry history of a circuit

No concurrency gurantees are provided for getting status. It's possible that status may represent a circuit in the middle of modification.

A storage backend may choose not to implement this method and instead return an empty array.

Each item in the history array is an array of two items (a tuple) of [run_time, succeeded], where run_time is a unix timestamp, and succeeded is a boolean, true if the run succeeded.

Raises:

  • (NotImplementedError)


161
162
163
# File 'lib/faulty/storage/interface.rb', line 161

def history(circuit)
  raise NotImplementedError
end

#listArray<String>

Get a list of all circuit names

If the storage backend does not support listing circuits, this may return an empty array.

Raises:

  • (NotImplementedError)


171
172
173
# File 'lib/faulty/storage/interface.rb', line 171

def list
  raise NotImplementedError
end

#lock(circuit, state) ⇒ void

This method returns an undefined value.

Lock the circuit in a given state

No concurrency gurantees are provided for locking

Raises:

  • (NotImplementedError)


109
110
111
# File 'lib/faulty/storage/interface.rb', line 109

def lock(circuit, state)
  raise NotImplementedError
end

#open(circuit, opened_at) ⇒ Boolean

Set the circuit state to open

If multiple parallel processes open the circuit simultaneously, open may be called more than once. If so, this method should return true only once, when the circuit transitions from closed to open.

If the backend does not support locking or atomic operations, then it may always return true, but that could result in duplicate open notifications.

If returning true, this method also updates opened_at to the current time.

Raises:

  • (NotImplementedError)


63
64
65
# File 'lib/faulty/storage/interface.rb', line 63

def open(circuit, opened_at)
  raise NotImplementedError
end

#reopen(circuit, opened_at, previous_opened_at) ⇒ Boolean

Reset the opened_at time for a half_open circuit

If multiple parallel processes open the circuit simultaneously, reopen may be called more than once. If so, this method should return true only once, when the circuit updates the opened_at value. It can use the value from previous_opened_at to do a compare-and-set operation.

If the backend does not support locking or atomic operations, then it may always return true, but that could result in duplicate reopen notifications.

Raises:

  • (NotImplementedError)


83
84
85
# File 'lib/faulty/storage/interface.rb', line 83

def reopen(circuit, opened_at, previous_opened_at)
  raise NotImplementedError
end

#reset(circuit) ⇒ void

This method returns an undefined value.

Reset the circuit to a fresh state

Clears all circuit status including entries, state, locks, opened_at, options, and any other values that would affect Status.

No concurrency gurantees are provided for resetting

Raises:

  • (NotImplementedError)


132
133
134
# File 'lib/faulty/storage/interface.rb', line 132

def reset(circuit)
  raise NotImplementedError
end

#set_options(circuit, stored_options) ⇒ void

This method returns an undefined value.

Store the options for a circuit

They should be returned exactly as given by #set_options

Raises:

  • (NotImplementedError)


28
29
30
# File 'lib/faulty/storage/interface.rb', line 28

def set_options(circuit, stored_options)
  raise NotImplementedError
end

#status(circuit) ⇒ Status

Get the status object for a circuit

No concurrency gurantees are provided for getting status. It's possible that status may represent a circuit in the middle of modification.

Raises:

  • (NotImplementedError)


143
144
145
# File 'lib/faulty/storage/interface.rb', line 143

def status(circuit)
  raise NotImplementedError
end

#unlock(circuit) ⇒ void

This method returns an undefined value.

Unlock the circuit from any state

No concurrency gurantees are provided for locking

Raises:

  • (NotImplementedError)


119
120
121
# File 'lib/faulty/storage/interface.rb', line 119

def unlock(circuit)
  raise NotImplementedError
end